﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TGC.RadioButtonUtil;

namespace TGC.RadioButtonUtil.TestForm
{
    public partial class  RadioBtnTestDialog : Form
    {

        private CharacterDescriptor descriptor = new CharacterDescriptor();
        private bool areOptionsSelected = false;

        public RadioBtnTestDialog()
        {
            InitializeComponent();

            skillPanel.DefaultOptionIndex = 1;

            crystalPanel.Load += new EventHandler(OptionPanels_Load);
            magicPanel.Load += new EventHandler(OptionPanels_Load);
            classPanel.Load +=  new EventHandler(OptionPanels_Load);
            skillPanel.Load += new EventHandler(OptionPanels_Load);

            crystalPanel.StateChanged += new EventHandler<RadioBtnContainerEventArgs>(crystalPanel_StateChanged);
            magicPanel.StateChanged += new EventHandler<RadioBtnContainerEventArgs>(OptionPanel_StateChanged);
            classPanel.StateChanged += new EventHandler<RadioBtnContainerEventArgs>(OptionPanel_StateChanged);
            skillPanel.StateChanged += new EventHandler<RadioBtnContainerEventArgs>(OptionPanel_StateChanged);
            chkBoxOutsider.CheckedChanged += new EventHandler(CheckBoxes_CheckedChanged);
            chkBoxSkillStonesOK.CheckedChanged += new EventHandler(CheckBoxes_CheckedChanged);

        }

        public CharacterDescriptor CharacterDescriptor
        {
            get
            {
                if (areOptionsSelected)
                    return descriptor;
                else
                    return null;
            }
        }

        private void OptionPanels_Load(object sender, EventArgs e)
        {
            if (object.ReferenceEquals(sender, crystalPanel))
                EnableValidMagicOptions(0);

            if (object.ReferenceEquals(sender, magicPanel))
            {
                skillPanel.Sort();
                skillPanel.GetRadioButtonAt(skillPanel.DefaultOptionIndex).Checked = true;
            }

            UpdateSelectionState();
            UpdateInfoPanel();
        }

        private void CheckBoxes_CheckedChanged(object sender, EventArgs e)
        {
            UpdateSelectionState();
            UpdateInfoPanel();
        }

        private void OptionPanel_StateChanged(object sender, RadioBtnContainerEventArgs e)
        {
            if (((RadioButtonContainer)sender).IsOptionSelected)
                UpdateSelectionState();
            
            UpdateInfoPanel();
        }

        private void crystalPanel_StateChanged(object sender, RadioBtnContainerEventArgs e)
        {
            if (e.IsOptionSelected)
            {
                EnableValidMagicOptions(e.Index);
                tabControl1.Enabled = true;
                UpdateSelectionState();
            }
            else
                DisableAllMagicOptions();

            UpdateInfoPanel();
        }

        private void DisableAllMagicOptions()
        {
            foreach (RadioButton rBtn in magicPanel.RadioButtons)
                rBtn.Enabled = false;
        }

        private void EnableValidMagicOptions(int crystalIndex)
        {
            /*
             * Rules:
             * Red crystal -->      fire magic & earth magic are available
             * Green crystal -->    earth magic is available
             * Blue crystal -->     ice, earth & fire magic are available
             */

            DisableAllMagicOptions();

            switch (crystalIndex)
            {   
                case 0:
                    // Red
                    magicPanel.GetRadioButtonAt(1).Enabled = true;
                    magicPanel.GetRadioButtonAt(2).Enabled = true;
                    if (magicPanel.GetRadioButtonAt(0).Checked)
                        magicPanel.GetRadioButtonAt(1).Checked = true;
                    break;
                case 1:
                    // Green
                    magicPanel.GetRadioButtonAt(1).Enabled = true;
                    magicPanel.GetRadioButtonAt(1).Checked = true;
                    break;
                default:
                    // Blue
                    magicPanel.GetRadioButtonAt(0).Enabled = true;
                    magicPanel.GetRadioButtonAt(1).Enabled = true;
                    magicPanel.GetRadioButtonAt(2).Enabled = true;
                    break;
            }
        }

        private void UpdateSelectionState()
        {
            descriptor.Crystal = (Crystal)crystalPanel.CurrentOptionIndex;
            descriptor.Magic = (Magic)magicPanel.CurrentOptionIndex;
            descriptor.Class = (Class)classPanel.CurrentOptionIndex;
            descriptor.IsOutsider = chkBoxOutsider.Checked;
            descriptor.Skill = skillPanel.CurrentOptionIndex + 1;
            descriptor.UseSkillStones = chkBoxSkillStonesOK.Checked;

            areOptionsSelected = true;
        }

        private void UpdateInfoPanel()
        {
            if (areOptionsSelected == false)
                lblInfo.Text = "[Nothing selected] Please select character options...";
            else
                lblInfo.Text = descriptor.ToString();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            crystalPanel.DeselectAllOptions();
            DisableAllMagicOptions();
            tabControl1.Enabled = false;
            areOptionsSelected = false;
            UpdateInfoPanel();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            areOptionsSelected = false;
            Close();
        }

        

    }
}
